Class: Synvert::Core::Rewriter::Instance
- Inherits:
-
Object
- Object
- Synvert::Core::Rewriter::Instance
- Includes:
- Helper
- Defined in:
- lib/synvert/core/rewriter/instance.rb
Overview
Instance Attribute Summary collapse
-
#current_node ⇒ Object
Current ast node.
-
#file_path ⇒ Object
readonly
File path.
-
#mutation_adapter ⇒ Object
readonly
Returns the value of attribute mutation_adapter.
Instance Method Summary collapse
-
#add_action(action) ⇒ Object
Add a custom action.
-
#add_leading_spaces(str, tab_size: 1) ⇒ String
Add leading spaces before the str according to Configuration.tab_width.
-
#append(code) ⇒ Object
It appends the code to the bottom of current node body.
-
#delete(*selectors, and_comma: false) ⇒ Object
It deletes child nodes.
-
#goto_node(child_node_name, &block) ⇒ Object
It creates a GotoScope to go to a child node, then continue operating on the child node.
-
#if_exist_node(nql_or_rules, &block) ⇒ Object
It creates a IfExistCondition to check if matching nodes exist in the child nodes, if so, then continue operating on each matching ast node.
-
#if_only_exist_node(nql_or_rules, &block) ⇒ Object
It creates a IfOnlyExistCondition to check if current node has only one child node and the child node matches, if so, then continue operating on each matching ast node.
-
#initialize(rewriter, file_path) { ... } ⇒ Instance
constructor
Initialize an Instance.
-
#insert(code, at: 'end', to: nil, and_comma: false) ⇒ Object
It inserts code.
-
#insert_after(code, to: nil, and_comma: false) ⇒ Object
It inserts the code next to the current node.
-
#insert_before(code, to: nil, and_comma: false) ⇒ Object
It inserts the code previous to the current node.
-
#node ⇒ Parser::AST::Node
Gets current node, it allows to get current node in block code.
-
#noop ⇒ Object
No operation.
-
#prepend(code) ⇒ Object
It prepends the code to the top of current node body.
-
#process ⇒ Object
Process the instance.
-
#process_with_node(node) { ... } ⇒ Object
Set current_node to node and process.
-
#process_with_other_node(node) { ... } ⇒ Object
Set current_node properly, process and set current_node back to original current_node.
-
#remove(and_comma: false) ⇒ Object
It removes current node.
-
#replace(*selectors, with:) ⇒ Object
It replaces the code of specified child nodes.
-
#replace_erb_stmt_with_expr ⇒ Object
It replaces erb stmt code to expr code.
-
#replace_with(code) ⇒ Object
It replaces the whole code of current node.
-
#test ⇒ Object
Test the instance.
-
#unless_exist_node(nql_or_rules, &block) ⇒ Object
It creates a UnlessExistCondition to check if matching nodes doesn't exist in the child nodes, if so, then continue operating on each matching ast node.
-
#warn(message) ⇒ Object
It creates a Warning to save warning message.
-
#within_node(nql_or_rules, options = {}) { ... } ⇒ Object
(also: #with_node, #find_node)
It creates a WithinScope to recursively find matching ast nodes, then continue operating on each matching ast node.
-
#wrap(with:) ⇒ Object
It wraps current node with code.
-
#wrap_with_quotes(str) ⇒ String
Wrap str string with single or doulbe quotes based on Configuration.single_quote.
Methods included from Helper
#add_arguments_with_parenthesis_if_necessary, #add_curly_brackets_if_necessary, #add_receiver_if_necessary, #reject_keys_from_hash, #strip_brackets
Constructor Details
#initialize(rewriter, file_path) { ... } ⇒ Instance
Initialize an Instance.
16 17 18 19 20 21 22 23 24 25 26 27 |
# File 'lib/synvert/core/rewriter/instance.rb', line 16 def initialize(rewriter, file_path, &block) @rewriter = rewriter @actions = [] @file_path = file_path @block = block strategy = NodeMutation::Strategy::KEEP_RUNNING if rewriter.[:strategy] == Strategy::ALLOW_INSERT_AT_SAME_POSITION strategy |= NodeMutation::Strategy::ALLOW_INSERT_AT_SAME_POSITION end NodeMutation.configure({ strategy: strategy, tab_width: Configuration.tab_width }) rewriter.helpers.each { |helper| singleton_class.send(:define_method, helper[:name], &helper[:block]) } end |
Instance Attribute Details
#current_node ⇒ Object
Returns current ast node.
35 |
# File 'lib/synvert/core/rewriter/instance.rb', line 35 attr_reader :file_path, :current_node, :mutation_adapter |
#file_path ⇒ Object (readonly)
Returns file path.
35 36 37 |
# File 'lib/synvert/core/rewriter/instance.rb', line 35 def file_path @file_path end |
#mutation_adapter ⇒ Object (readonly)
Returns the value of attribute mutation_adapter.
35 |
# File 'lib/synvert/core/rewriter/instance.rb', line 35 attr_reader :file_path, :current_node, :mutation_adapter |
Instance Method Details
#add_action(action) ⇒ Object
Add a custom action.
388 389 390 |
# File 'lib/synvert/core/rewriter/instance.rb', line 388 def add_action(action) @current_mutation.actions << action.process end |
#add_leading_spaces(str, tab_size: 1) ⇒ String
Add leading spaces before the str according to Configuration.tab_width.
420 421 422 |
# File 'lib/synvert/core/rewriter/instance.rb', line 420 def add_leading_spaces(str, tab_size: 1) (" " * Configuration.tab_width * tab_size) + str; end |
#append(code) ⇒ Object
It appends the code to the bottom of current node body.
220 221 222 |
# File 'lib/synvert/core/rewriter/instance.rb', line 220 def append(code) @current_mutation.append(@current_node, code) end |
#delete(*selectors, and_comma: false) ⇒ Object
It deletes child nodes.
357 358 359 |
# File 'lib/synvert/core/rewriter/instance.rb', line 357 def delete(*selectors, and_comma: false) @current_mutation.delete(@current_node, *selectors, and_comma: and_comma) end |
#goto_node(child_node_name, &block) ⇒ Object
It creates a GotoScope to go to a child node, then continue operating on the child node.
159 160 161 |
# File 'lib/synvert/core/rewriter/instance.rb', line 159 def goto_node(child_node_name, &block) Rewriter::GotoScope.new(self, child_node_name, &block).process end |
#if_exist_node(nql_or_rules, &block) ⇒ Object
It creates a Synvert::Core::Rewriter::IfExistCondition to check if matching nodes exist in the child nodes, if so, then continue operating on each matching ast node.
173 174 175 |
# File 'lib/synvert/core/rewriter/instance.rb', line 173 def if_exist_node(nql_or_rules, &block) Rewriter::IfExistCondition.new(self, nql_or_rules, &block).process end |
#if_only_exist_node(nql_or_rules, &block) ⇒ Object
It creates a Synvert::Core::Rewriter::IfOnlyExistCondition to check if current node has only one child node and the child node matches, if so, then continue operating on each matching ast node.
202 203 204 |
# File 'lib/synvert/core/rewriter/instance.rb', line 202 def if_only_exist_node(nql_or_rules, &block) Rewriter::IfOnlyExistCondition.new(self, nql_or_rules, &block).process end |
#insert(code, at: 'end', to: nil, and_comma: false) ⇒ Object
It inserts code.
254 255 256 |
# File 'lib/synvert/core/rewriter/instance.rb', line 254 def insert(code, at: 'end', to: nil, and_comma: false) @current_mutation.insert(@current_node, code, at: at, to: to, and_comma: and_comma) end |
#insert_after(code, to: nil, and_comma: false) ⇒ Object
It inserts the code next to the current node.
270 271 272 273 |
# File 'lib/synvert/core/rewriter/instance.rb', line 270 def insert_after(code, to: nil, and_comma: false) column = ' ' * NodeMutation.adapter.get_start_loc(@current_node, to).column @current_mutation.insert(@current_node, "\n#{column}#{code}", at: 'end', to: to, and_comma: and_comma) end |
#insert_before(code, to: nil, and_comma: false) ⇒ Object
It inserts the code previous to the current node.
287 288 289 290 |
# File 'lib/synvert/core/rewriter/instance.rb', line 287 def insert_before(code, to: nil, and_comma: false) column = ' ' * NodeMutation.adapter.get_start_loc(@current_node, to).column @current_mutation.insert(@current_node, "#{code}\n#{column}", at: 'beginning', to: to, and_comma: and_comma) end |
#node ⇒ Parser::AST::Node
Gets current node, it allows to get current node in block code.
96 97 98 |
# File 'lib/synvert/core/rewriter/instance.rb', line 96 def node @current_node end |
#noop ⇒ Object
No operation.
379 380 381 |
# File 'lib/synvert/core/rewriter/instance.rb', line 379 def noop @current_mutation.noop(@current_node) end |
#prepend(code) ⇒ Object
It prepends the code to the top of current node body.
238 239 240 |
# File 'lib/synvert/core/rewriter/instance.rb', line 238 def prepend(code) @current_mutation.prepend(@current_node, code) end |
#process ⇒ Object
Process the instance. It finds specified files, for each file, it executes the block code, rewrites the original code, then writes the code back to the original file.
41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 |
# File 'lib/synvert/core/rewriter/instance.rb', line 41 def process puts @file_path if Configuration.show_run_process absolute_file_path = File.join(Configuration.root_path, @file_path) while true source = read_source(absolute_file_path) @current_mutation = NodeMutation.new(source) @mutation_adapter = NodeMutation.adapter begin node = parse_code(@file_path, source) process_with_node(node) do instance_eval(&@block) end result = @current_mutation.process if result.affected? @rewriter.add_affected_file(file_path) write_source(absolute_file_path, result.new_source) end break unless result.conflicted? rescue Parser::SyntaxError puts "[Warn] file #{file_path} was not parsed correctly." # do nothing, iterate next file end end end |
#process_with_node(node) { ... } ⇒ Object
Set current_node to node and process.
104 105 106 107 108 |
# File 'lib/synvert/core/rewriter/instance.rb', line 104 def process_with_node(node) self.current_node = node yield self.current_node = node end |
#process_with_other_node(node) { ... } ⇒ Object
Set current_node properly, process and set current_node back to original current_node.
114 115 116 117 118 119 |
# File 'lib/synvert/core/rewriter/instance.rb', line 114 def process_with_other_node(node) original_node = current_node self.current_node = node yield self.current_node = original_node end |
#remove(and_comma: false) ⇒ Object
It removes current node.
343 344 345 |
# File 'lib/synvert/core/rewriter/instance.rb', line 343 def remove(and_comma: false) @current_mutation.remove(@current_node, and_comma: and_comma) end |
#replace(*selectors, with:) ⇒ Object
It replaces the code of specified child nodes.
333 334 335 |
# File 'lib/synvert/core/rewriter/instance.rb', line 333 def replace(*selectors, with:) @current_mutation.replace(@current_node, *selectors, with: with) end |
#replace_erb_stmt_with_expr ⇒ Object
It replaces erb stmt code to expr code.
302 303 304 305 306 307 |
# File 'lib/synvert/core/rewriter/instance.rb', line 302 def replace_erb_stmt_with_expr absolute_file_path = File.join(Configuration.root_path, @file_path) erb_source = read_source(absolute_file_path) action = Rewriter::ReplaceErbStmtWithExprAction.new(@current_node, erb_source) add_action(action) end |
#replace_with(code) ⇒ Object
It replaces the whole code of current node.
318 319 320 |
# File 'lib/synvert/core/rewriter/instance.rb', line 318 def replace_with(code) @current_mutation.replace_with(@current_node, code) end |
#test ⇒ Object
Test the instance. It finds specified files, for each file, it executes the block code, tests the original code, then returns the actions.
72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 |
# File 'lib/synvert/core/rewriter/instance.rb', line 72 def test absolute_file_path = File.join(Configuration.root_path, file_path) source = read_source(absolute_file_path) @current_mutation = NodeMutation.new(source) @mutation_adapter = NodeMutation.adapter begin node = parse_code(file_path, source) process_with_node(node) do instance_eval(&@block) end result = @current_mutation.test result.file_path = file_path result rescue Parser::SyntaxError puts "[Warn] file #{file_path} was not parsed correctly." # do nothing, iterate next file end end |
#unless_exist_node(nql_or_rules, &block) ⇒ Object
It creates a UnlessExistCondition to check if matching nodes doesn't exist in the child nodes, if so, then continue operating on each matching ast node.
187 188 189 |
# File 'lib/synvert/core/rewriter/instance.rb', line 187 def unless_exist_node(nql_or_rules, &block) Rewriter::UnlessExistCondition.new(self, nql_or_rules, &block).process end |
#warn(message) ⇒ Object
It creates a Warning to save warning message.
398 399 400 |
# File 'lib/synvert/core/rewriter/instance.rb', line 398 def warn() @rewriter.add_warning Rewriter::Warning.new(self, ) end |
#within_node(nql_or_rules, options = {}) { ... } ⇒ Object Also known as: with_node, find_node
It creates a WithinScope to recursively find matching ast nodes, then continue operating on each matching ast node.
140 141 142 143 144 |
# File 'lib/synvert/core/rewriter/instance.rb', line 140 def within_node(nql_or_rules, = {}, &block) Rewriter::WithinScope.new(self, nql_or_rules, , &block).process rescue NodeQueryLexer::ScanError, Racc::ParseError => e raise NodeQuery::Compiler::ParseError, "Invalid query string: #{nql_or_rules}" end |
#wrap(with:) ⇒ Object
It wraps current node with code.
374 375 376 |
# File 'lib/synvert/core/rewriter/instance.rb', line 374 def wrap(with:) @current_mutation.wrap(@current_node, with: with) end |
#wrap_with_quotes(str) ⇒ String
Wrap str string with single or doulbe quotes based on Configuration.single_quote.
405 406 407 408 409 410 411 412 413 414 |
# File 'lib/synvert/core/rewriter/instance.rb', line 405 def wrap_with_quotes(str) quote = Configuration.single_quote ? "'" : '"'; another_quote = Configuration.single_quote ? '"' : "'"; if str.include?(quote) && !str.include?(another_quote) return "#{another_quote}#{str}#{another_quote}" end escaped_str = str.gsub(quote) { |_char| '\\' + quote } quote + escaped_str + quote end |