Class: Synvert::Core::Rewriter::Instance
- Inherits:
-
Object
- Object
- Synvert::Core::Rewriter::Instance
- Includes:
- Helper
- Defined in:
- lib/synvert/core/rewriter/instance.rb
Overview
Instance Attribute Summary collapse
-
#current_node ⇒ Object
Current ast node.
-
#file_path ⇒ Object
readonly
File path.
-
#mutation_adapter ⇒ Object
readonly
Returns the value of attribute mutation_adapter.
Instance Method Summary collapse
-
#add_action(action) ⇒ Object
Add a custom action.
-
#append(code) ⇒ Object
It appends the code to the bottom of current node body.
-
#delete(*selectors, **options) ⇒ Object
It deletes child nodes.
-
#goto_node(child_node_name, &block) ⇒ Object
It creates a GotoScope to go to a child node, then continue operating on the child node.
-
#if_exist_node(nql_or_rules, &block) ⇒ Object
It creates a IfExistCondition to check if matching nodes exist in the child nodes, if so, then continue operating on each matching ast node.
-
#if_only_exist_node(nql_or_rules, &block) ⇒ Object
It creates a IfOnlyExistCondition to check if current node has only one child node and the child node matches, if so, then continue operating on each matching ast node.
-
#initialize(rewriter, file_path) { ... } ⇒ Instance
constructor
Initialize an Instance.
-
#insert(code, at: 'end', to: nil) ⇒ Object
It inserts code.
-
#insert_after(code, to: nil) ⇒ Object
It inserts the code next to the current node.
-
#insert_before(code, to: nil) ⇒ Object
It inserts the code previous to the current node.
-
#node ⇒ Parser::AST::Node
Gets current node, it allows to get current node in block code.
-
#noop ⇒ Object
No operation.
-
#prepend(code) ⇒ Object
It prepends the code to the top of current node body.
-
#process ⇒ Object
Process the instance.
-
#process_with_node(node) { ... } ⇒ Object
Set current_node to node and process.
-
#process_with_other_node(node) { ... } ⇒ Object
Set current_node properly, process and set current_node back to original current_node.
-
#remove(**options) ⇒ Object
It removes current node.
-
#replace(*selectors, with:) ⇒ Object
It replaces the code of specified child nodes.
-
#replace_erb_stmt_with_expr ⇒ Object
It replaces erb stmt code to expr code.
-
#replace_with(code) ⇒ Object
It replaces the whole code of current node.
-
#test ⇒ Object
Test the instance.
-
#unless_exist_node(nql_or_rules, &block) ⇒ Object
It creates a UnlessExistCondition to check if matching nodes doesn't exist in the child nodes, if so, then continue operating on each matching ast node.
-
#warn(message) ⇒ Object
It creates a Warning to save warning message.
-
#within_node(nql_or_rules, options = {}) { ... } ⇒ Object
(also: #with_node, #find_node)
It creates a WithinScope to recursively find matching ast nodes, then continue operating on each matching ast node.
-
#wrap(with:) ⇒ Object
It wraps current node with code.
-
#wrap_with_quotes(str) ⇒ String
Wrap str string with single or doulbe quotes based on Configuration.single_quote.
Methods included from Helper
#add_arguments_with_parenthesis_if_necessary, #add_curly_brackets_if_necessary, #add_receiver_if_necessary, #reject_keys_from_hash, #strip_brackets
Constructor Details
#initialize(rewriter, file_path) { ... } ⇒ Instance
Initialize an Instance.
16 17 18 19 20 21 22 23 24 25 26 27 |
# File 'lib/synvert/core/rewriter/instance.rb', line 16 def initialize(rewriter, file_path, &block) @rewriter = rewriter @actions = [] @file_path = file_path @block = block strategy = NodeMutation::Strategy::KEEP_RUNNING if rewriter.[:strategy] == Strategy::ALLOW_INSERT_AT_SAME_POSITION strategy |= NodeMutation::Strategy::ALLOW_INSERT_AT_SAME_POSITION end NodeMutation.configure({ strategy: strategy }) rewriter.helpers.each { |helper| singleton_class.send(:define_method, helper[:name], &helper[:block]) } end |
Instance Attribute Details
#current_node ⇒ Object
Returns current ast node.
35 |
# File 'lib/synvert/core/rewriter/instance.rb', line 35 attr_reader :file_path, :current_node, :mutation_adapter |
#file_path ⇒ Object (readonly)
Returns file path.
35 36 37 |
# File 'lib/synvert/core/rewriter/instance.rb', line 35 def file_path @file_path end |
#mutation_adapter ⇒ Object (readonly)
Returns the value of attribute mutation_adapter.
35 |
# File 'lib/synvert/core/rewriter/instance.rb', line 35 attr_reader :file_path, :current_node, :mutation_adapter |
Instance Method Details
#add_action(action) ⇒ Object
Add a custom action.
399 400 401 |
# File 'lib/synvert/core/rewriter/instance.rb', line 399 def add_action(action) @current_mutation.actions << action.process end |
#append(code) ⇒ Object
It appends the code to the bottom of current node body.
237 238 239 |
# File 'lib/synvert/core/rewriter/instance.rb', line 237 def append(code) @current_mutation.append(@current_node, code) end |
#delete(*selectors, **options) ⇒ Object
It deletes child nodes.
368 369 370 |
# File 'lib/synvert/core/rewriter/instance.rb', line 368 def delete(*selectors, **) @current_mutation.delete(@current_node, *selectors, **) end |
#goto_node(child_node_name, &block) ⇒ Object
It creates a GotoScope to go to a child node, then continue operating on the child node.
176 177 178 |
# File 'lib/synvert/core/rewriter/instance.rb', line 176 def goto_node(child_node_name, &block) Rewriter::GotoScope.new(self, child_node_name, &block).process end |
#if_exist_node(nql_or_rules, &block) ⇒ Object
It creates a Synvert::Core::Rewriter::IfExistCondition to check if matching nodes exist in the child nodes, if so, then continue operating on each matching ast node.
190 191 192 |
# File 'lib/synvert/core/rewriter/instance.rb', line 190 def if_exist_node(nql_or_rules, &block) Rewriter::IfExistCondition.new(self, nql_or_rules, &block).process end |
#if_only_exist_node(nql_or_rules, &block) ⇒ Object
It creates a Synvert::Core::Rewriter::IfOnlyExistCondition to check if current node has only one child node and the child node matches, if so, then continue operating on each matching ast node.
219 220 221 |
# File 'lib/synvert/core/rewriter/instance.rb', line 219 def if_only_exist_node(nql_or_rules, &block) Rewriter::IfOnlyExistCondition.new(self, nql_or_rules, &block).process end |
#insert(code, at: 'end', to: nil) ⇒ Object
It inserts code.
270 271 272 |
# File 'lib/synvert/core/rewriter/instance.rb', line 270 def insert(code, at: 'end', to: nil) @current_mutation.insert(@current_node, code, at: at, to: to) end |
#insert_after(code, to: nil) ⇒ Object
It inserts the code next to the current node.
284 285 286 287 |
# File 'lib/synvert/core/rewriter/instance.rb', line 284 def insert_after(code, to: nil) column = ' ' * NodeMutation.adapter.get_start_loc(@current_node).column @current_mutation.insert(@current_node, "\n#{column}#{code}", at: 'end', to: to) end |
#insert_before(code, to: nil) ⇒ Object
It inserts the code previous to the current node.
299 300 301 302 |
# File 'lib/synvert/core/rewriter/instance.rb', line 299 def insert_before(code, to: nil) column = ' ' * NodeMutation.adapter.get_start_loc(@current_node).column @current_mutation.insert(@current_node, "#{code}\n#{column}", at: 'beginning', to: to) end |
#node ⇒ Parser::AST::Node
Gets current node, it allows to get current node in block code.
113 114 115 |
# File 'lib/synvert/core/rewriter/instance.rb', line 113 def node @current_node end |
#noop ⇒ Object
No operation.
390 391 392 |
# File 'lib/synvert/core/rewriter/instance.rb', line 390 def noop @current_mutation.noop(@current_node) end |
#prepend(code) ⇒ Object
It prepends the code to the top of current node body.
255 256 257 |
# File 'lib/synvert/core/rewriter/instance.rb', line 255 def prepend(code) @current_mutation.prepend(@current_node, code) end |
#process ⇒ Object
Process the instance. It finds specified files, for each file, it executes the block code, rewrites the original code, then writes the code back to the original file.
41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 |
# File 'lib/synvert/core/rewriter/instance.rb', line 41 def process puts @file_path if Configuration.show_run_process absolute_file_path = File.join(Configuration.root_path, @file_path) while true source = read_source(absolute_file_path) @current_mutation = NodeMutation.new(source) @mutation_adapter = NodeMutation.adapter begin node = parse_code(@file_path, source) process_with_node(node) do instance_eval(&@block) rescue NoMethodError => e puts [ "error: #{e.message}", "file: #{file_path}", "source: #{source}", "line: #{current_node.line}" ].join("\n") raise end result = @current_mutation.process if result.affected? @rewriter.add_affected_file(file_path) write_source(absolute_file_path, result.new_source) end break unless result.conflicted? rescue Parser::SyntaxError puts "[Warn] file #{file_path} was not parsed correctly." # do nothing, iterate next file end end end |
#process_with_node(node) { ... } ⇒ Object
Set current_node to node and process.
121 122 123 124 125 |
# File 'lib/synvert/core/rewriter/instance.rb', line 121 def process_with_node(node) self.current_node = node yield self.current_node = node end |
#process_with_other_node(node) { ... } ⇒ Object
Set current_node properly, process and set current_node back to original current_node.
131 132 133 134 135 136 |
# File 'lib/synvert/core/rewriter/instance.rb', line 131 def process_with_other_node(node) original_node = current_node self.current_node = node yield self.current_node = original_node end |
#remove(**options) ⇒ Object
It removes current node.
353 354 355 |
# File 'lib/synvert/core/rewriter/instance.rb', line 353 def remove(**) @current_mutation.remove(@current_node, **) end |
#replace(*selectors, with:) ⇒ Object
It replaces the code of specified child nodes.
342 343 344 |
# File 'lib/synvert/core/rewriter/instance.rb', line 342 def replace(*selectors, with:) @current_mutation.replace(@current_node, *selectors, with: with) end |
#replace_erb_stmt_with_expr ⇒ Object
It replaces erb stmt code to expr code.
314 315 316 |
# File 'lib/synvert/core/rewriter/instance.rb', line 314 def replace_erb_stmt_with_expr @current_mutation.actions << Rewriter::ReplaceErbStmtWithExprAction.new(@current_node).process end |
#replace_with(code) ⇒ Object
It replaces the whole code of current node.
327 328 329 |
# File 'lib/synvert/core/rewriter/instance.rb', line 327 def replace_with(code) @current_mutation.replace_with(@current_node, code) end |
#test ⇒ Object
Test the instance. It finds specified files, for each file, it executes the block code, tests the original code, then returns the actions.
80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 |
# File 'lib/synvert/core/rewriter/instance.rb', line 80 def test absolute_file_path = File.join(Configuration.root_path, file_path) source = read_source(absolute_file_path) @current_mutation = NodeMutation.new(source) @mutation_adapter = NodeMutation.adapter begin node = parse_code(file_path, source) process_with_node(node) do instance_eval(&@block) rescue NoMethodError => e puts [ "error: #{e.message}", "file: #{file_path}", "source: #{source}", "line: #{current_node.line}" ].join("\n") raise end result = @current_mutation.test result.file_path = file_path result rescue Parser::SyntaxError puts "[Warn] file #{file_path} was not parsed correctly." # do nothing, iterate next file end end |
#unless_exist_node(nql_or_rules, &block) ⇒ Object
It creates a UnlessExistCondition to check if matching nodes doesn't exist in the child nodes, if so, then continue operating on each matching ast node.
204 205 206 |
# File 'lib/synvert/core/rewriter/instance.rb', line 204 def unless_exist_node(nql_or_rules, &block) Rewriter::UnlessExistCondition.new(self, nql_or_rules, &block).process end |
#warn(message) ⇒ Object
It creates a Warning to save warning message.
409 410 411 |
# File 'lib/synvert/core/rewriter/instance.rb', line 409 def warn() @rewriter.add_warning Rewriter::Warning.new(self, ) end |
#within_node(nql_or_rules, options = {}) { ... } ⇒ Object Also known as: with_node, find_node
It creates a WithinScope to recursively find matching ast nodes, then continue operating on each matching ast node.
157 158 159 160 161 |
# File 'lib/synvert/core/rewriter/instance.rb', line 157 def within_node(nql_or_rules, = {}, &block) Rewriter::WithinScope.new(self, nql_or_rules, , &block).process rescue NodeQueryLexer::ScanError, Racc::ParseError => e raise NodeQuery::Compiler::ParseError, "Invalid query string: #{nql_or_rules}" end |
#wrap(with:) ⇒ Object
It wraps current node with code.
385 386 387 |
# File 'lib/synvert/core/rewriter/instance.rb', line 385 def wrap(with:) @current_mutation.wrap(@current_node, with: with) end |
#wrap_with_quotes(str) ⇒ String
Wrap str string with single or doulbe quotes based on Configuration.single_quote.
416 417 418 419 420 421 422 423 424 |
# File 'lib/synvert/core/rewriter/instance.rb', line 416 def wrap_with_quotes(str) quote = Configuration.single_quote ? "'" : '"'; another_quote = Configuration.single_quote ? '"' : "'"; if str.include?(quote) && !str.include?(another_quote) return "#{another_quote}#{str}#{another_quote}" end escaped_str = str.gsub(quote) { |char| '\\' + quote } quote + escaped_str + quote end |